package weather.model;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.SortedSet;

public class GlobalForecast implements Forecast {
	private String[] weatherNames = { "NESSUNO", "neve", "tempesta", "pioggia", 
									  "nebbia", "nuvoloso", "variabile", "sereno" };
	private String place, weather;
	private Date date;
	private float tempMin, tempMax;

	private static void clearHMS(Calendar cal) {
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MILLISECOND, 0);
	}

	public static boolean sameDate(Date d1, Date d2) {
		GregorianCalendar cal = new GregorianCalendar();

		cal.setTime(d1);
		clearHMS(cal);
		long td1 = cal.getTimeInMillis();

		cal.setTime(d2);
		clearHMS(cal);
		long td2 = cal.getTimeInMillis();

		return td1 == td2;
	}

	public static Collection<GlobalForecast> calcGlobalForecast(
			SortedSet<PunctualForecast> forecasts) {
		ArrayList<GlobalForecast> retval = new ArrayList<GlobalForecast>();
		ArrayList<PunctualForecast> pfList = new ArrayList<PunctualForecast>();
		Date lastDate = null;

		for (PunctualForecast pf : forecasts) {
			if (lastDate == null) {
				lastDate = pf.getDate();
			} else {
				if (!sameDate(pf.getDate(), lastDate)) {
					retval.add(new GlobalForecast(pfList));
					pfList.clear();
					lastDate = pf.getDate();
				}
			}
			pfList.add(pf);
		}

		if (pfList.size() > 0) {
			retval.add(new GlobalForecast(pfList));
		}

		return retval;
	}

	private GlobalForecast(Collection<PunctualForecast> forecasts) {

		if ((forecasts == null) || (forecasts.size() == 0)) {
			throw new RuntimeException(
					"Impossibile creare il GlobalForecast: lista in input vuota");
		}

		ArrayList<PunctualForecast> forecastList = (ArrayList<PunctualForecast>) forecasts;
		PunctualForecast pf = forecastList.get(0);

		GregorianCalendar cal = new GregorianCalendar();
		place = pf.getPlace();
		cal.setTime(pf.getDate());
		clearHMS(cal);
		date = cal.getTime();

		tempMin = pf.getTemperature();
		tempMax = pf.getTemperature();

		int sum = 0;
		int sumDur = 0;

		for (int i = 0; i < forecastList.size(); i++) {
			PunctualForecast pfc = forecastList.get(i);

			float temp = pfc.getTemperature();
			if (temp > tempMax) {
				tempMax = temp;
			}
			if (temp < tempMin) {
				tempMin = temp;
			}

			int weatherWeight = getWeatherWeight(pfc.getWeather());
			long duration;
			if (i == forecastList.size() - 1) {
				cal.setTime(pfc.getDate());
				clearHMS(cal); // calendar set at 0:00 of last forecast date
				long midnight = (24 * 3600 * 1000) + cal.getTimeInMillis();
				duration = midnight - pfc.getDate().getTime();
			} else {
				duration = forecastList.get(i+1).getDate().getTime() - pfc.getDate().getTime();
			}

			sum += weatherWeight * duration;
			sumDur += duration;
		}

		int average = Math.round((float) sum / sumDur);
		this.weather = weatherNames[average];
	}

	private int getWeatherWeight(String val) {
		for (int i = 1; i < weatherNames.length; i++)
			if (weatherNames[i].equalsIgnoreCase(val))
				return i;
		return 0; // should never arrive here...
	}

	public String getPlace() {
		return place;
	}

	@Override
	public Date getDate() {
		return date;
	}

	public String getWeather() {
		return weather;
	}

	public float getMinTemp() {
		return tempMin;
	}

	public float getMaxTemp() {
		return tempMax;
	}

	@Override
	public String toString() {
		DateFormat dateFormatter = DateFormat.getDateInstance(DateFormat.SHORT);

		return place + " " + dateFormatter.format(date)
				+ " temperatura minima: " + tempMin + ""
				+ " temperatura massima: " + tempMax + ""
				+ " tempo previsto: " + weather + "\n";
	}

}
